package TKL;

use XML::LibXML;
use TKL::File;
use TKL::User;
use Carp;
use Cwd;
use strict;

our $xml_header = "<?xml version=\"1.0\" encoding=\"ISO-8859-1\" standalone=\"yes\"?>";
our $tkl_suffix = "tkl";
our $schema_suffix = "xsd";
our $userlist = "users.tkl";
our $schema_NS_URI = "http://www.w3.org/2001/XMLSchema";
our $uri_element_type = "xs:anyURI";
our $VERSION = "1.4.2";
our $ABSTRACT = "Base class for the TKL perl packages.";

sub new {
    my ($class, @rest) = @_;
    my %args = (tkl_config	=> "tkl.config",
    		oai_dir		=> "oai",
		schema_dir	=> "schemas",
		@rest);
    my $self = \%args;

    bless $self, $class;
    $self->find_portal_root unless defined($self->{root});
    $self->{libxml} = new XML::LibXML;

    return $self;
}


sub dom {
    return shift->{libxml};
}


sub document_element {
    my ($self, $xml_file) = @_;
    my $doc = $self->dom->parse_file($xml_file);

    return $doc->documentElement;
}
    

sub check_path {
    my ($self, $path) = @_;

    if ($ENV{MOD_PERL}) {	## If running in MOD_PERL environment, use Apache docroot
	$path = $ENV{DOCUMENT_ROOT} . "/$path";
    }
    $path = $self->normalize_file($path);
    my @steps = split /\//, $path;
    while (@steps) {
	my $root_cand = join "/", @steps;
	my $tkl_config = "$root_cand/tkl.config";
	if (-f $tkl_config) {
	    return $root_cand;
	}
	pop @steps;
    }
    return undef;
}


sub tkl_die {
    my ($self, @msg) = @_;

    print STDERR "Fatal error: $0\n\n";
    foreach (@msg) {
	print STDERR "$_\n";
    }
    exit(1);
}


sub find_portal_root {
    my $self = shift;
    
    return $self->{root} if defined($self->{root});
    if (defined($self->{path})) {
	my $root = $self->check_path($self->{path});
	if (defined($root)) {
	    return $self->{root} = $root;
	} else {
	    $self->tkl_die("Out of portal scope: '" . $self->{path} . "'\n\n");
	}
    }
    my $pwd = cwd;
    my @path = split /\//, $pwd;
    while (@path) {
	my $super_dir = join "/", @path;
	my $file = "$super_dir/" . $self->{tkl_config};
	if (-f $file) {
	    return $self->{root} = $super_dir;
	}
	pop(@path);
    }
    $self->tkl_die("Unable to find portal root");
}
    
sub get_user_info {
    my ($self, $user) = @_;
    my $root = $self->find_portal_root;
    my $user_file = "$root/$userlist";
    my $users = $self->{parsed_users};

    unless (defined($users)) {
	unless (-f $user_file) {
	    $self->tkl_die("Unable to find user file '$user_file'");
	}
	$users = $self->{parsed_users} = new TKL::File(file => $user_file, root => $root);
    }

    my $user_doc_elem = $users->document_element;
    my ($user_node) = $user_doc_elem->findnodes("/users/user[login='$user']");
    if (defined($user_node)) {
	return new TKL::User(user_node => $user_node, root => $self->find_portal_root);
    } else {
	return undef;
    }
}


sub xml_trim {
    my ($self, $cdata) = @_;

    $cdata =~ s/^\s+//g;
    $cdata =~ s/\s+$//g;
    $cdata =~ s/&amp;/&/g;
    $cdata =~ s/&/&amp;/g;
    $cdata =~ s/</&lt;/g;
    $cdata =~ s/>/&gt;/g;

    return $cdata;
}


sub dom2hash {
    my ($self, $dom, $href) = @_;
    my $name = $dom->nodeName();
    my $cdata = "";
    my $offset;
    
    if (ref($href->{$name}) eq 'ARRAY') {
	$offset = $#{$href->{$name}} + 1;
    } else {
	$href->{$name} = [];
	$offset = 0;
    }
    foreach my $kid ($dom->childNodes()) {
	my $type = $kid->getType();
	if ($type == XML_ELEMENT_NODE) {
	    my $subtree = $href->{$name}->[$offset]->{nodeset};
	    if (!defined($subtree)) {
		$subtree = $href->{$name}->[$offset]->{nodeset} = {};
	    }
	    $self->dom2hash($kid, $subtree);
	} elsif ($type == XML_TEXT_NODE) {
	    $cdata .= $kid->nodeValue();
	} else {
	    warn "$0 - dom2hash: Uknown node type '$type'";
	}
    }
    $href->{$name}->[$offset]->{content} = $self->xml_trim($cdata);
    if (my @attr = $dom->attributes) {
	foreach my $entry (@attr) {
	    $href->{$name}->[$offset]->{attr}->{$entry->nodeName()} = $entry->nodeValue();
	}
    }
}


sub parse {
    my ($self, $xml_file) = @_;
    my $root = $self->document_element($xml_file);

    $self->dom2hash($root, my $href = {});

    return $href;
}


sub normalize_file {
    my ($self, $file) = @_;
    
    $file =~ s/\/+/\//g;
    return $file;
}


sub get_file {
    my ($self, $file) = @_;
    my $root = $self->find_portal_root;
    my $abs_file = $self->normalize_file("$root/$file");

    return new TKL::File(root => $root, file => $abs_file);
}


sub locate_file {
    my ($self, $file) = @_;
    my $root = $self->find_portal_root;
    my $inside_portal = 1;

    $file = $self->normalize_file("$root/$file");

    my ($base, $f) = ($file =~ /(.*)\/([^\/]*)$/);

    do {
	my $tmp_file = "$base/$f";
	return $tmp_file if -e $tmp_file;
	if (-e "$base/tkl.config") {
	    $inside_portal = 0;
	}
	$base =~ s/\/[^\/]*$//;
    } while (length($base) && $inside_portal);

    return undef;
}
    

sub browse {
    my ($self, $dir) = @_;
    my $files = [];
    my $dirs = [];
    my $root = $self->{root};
    local *DIR;

    $dir = "" unless defined($dir);
    $dir = "$root/$dir/";
    opendir DIR, $dir or die "$0 - browse: Unable to open directory '$dir': $!";
    while (defined(my $entry = readdir DIR)) {
	next if $entry =~ /^\.\.?/;
	my $file = "$dir/$entry";
	if (-d $file) {
	    push @$dirs, $entry;
	} elsif ($entry =~ /\.$tkl_suffix$/) {
	    push @$files, new TKL::File(root => $root, file => $self->normalize_file($file));
	}
    }
    closedir DIR or warn "$0 - browse: Unable to close directory '$dir' properly: $!";
    
    return {files => $files, dirs => $dirs};
}



package XML::LibXML::Node;
use XML::LibXML;

## Here goes TKL extensions to the XML::LibXML package
## Methods should be appended with tkl_ to indicate their origin!

sub tkl_cdata {
    my $self = shift;
    my $cdata = "";
    my @children = $self->getChildnodes();

    foreach my $node (@children) {
	my $type = $node->getType();
	if ($type == XML_TEXT_NODE) {
	    $cdata .= $node->getData();
	} elsif ($type == XML_ELEMENT_NODE) {
	    $cdata .= $node->tkl_cdata();
	}
    }

    return $cdata;
}


1;

__END__

=head1 NAME

TKL - Base class of the TKL perl project.

=head1 SYNOPSIS

  use TKL;

  my $tkl = new TKL;

=head1 DESCRIPTION

=head1 AUTHOR

Anders Snderberg Mortensen <sondberg@indexdata.dk>
Indexdata
2003/06/12

=head1 SEE-ALSO

perl(1).

=cut

## $Log: TKL.pm,v $
## Revision 1.7  2003/10/08 12:55:14  sondberg
## Increased version number.
##
## Revision 1.6  2003/10/06 13:58:13  marc
## forgotten to update version in perl
##
## Revision 1.5  2003/09/29 08:07:25  sondberg
## Root path is explicitly passed when creating new TKL::User and TKL::File objects.
##
## Revision 1.4  2003/09/25 10:25:00  marc
## version updated to 1.4.0
##
## Revision 1.3  2003/09/05 09:57:00  marc
## version updated to 1.3.2
##
## Revision 1.2  2003/08/12 12:19:54  marc
## bug in oai harvester init script removed, upgraded to version 1.3.0
##
## Revision 1.1  2003/08/04 13:32:56  marc
## added perl code
##
## Revision 1.11  2003/08/01 14:44:26  marc
## version number updated to 1.2.7-1, tcl robot logmessages thinned out
##
## Revision 1.10  2003/07/22 10:28:48  sondberg
## Added root parm to TKL::File constructor.
##
## Revision 1.9  2003/07/05 22:05:16  sondberg
## Introduced methods tkl_die and locate_file.
##
## Revision 1.8  2003/07/05 15:48:25  sondberg
## Small changes/improvements, and added method get_file.
##
## Revision 1.7  2003/07/03 20:39:38  sondberg
## Allowed to specify a path member in the blessed hash, which is interpreted as
## a link into a portal area. The portal root is then extracted if possible.
##
## Revision 1.6  2003/06/18 11:56:09  sondberg
## Added method get_user_info, which returns an object blessed into the TKL::User package.
##
## Revision 1.5  2003/06/17 09:27:50  sondberg
## Added the extension tkl_cdata to the package XML::LibXML::Node.
##
## Revision 1.4  2003/06/16 14:23:39  marc
## HTTP::OAI moved to TKL::OAI
##
## Revision 1.3  2003/06/13 11:11:30  sondberg
## Added method normalize_path.
##
## Revision 1.2  2003/06/12 12:31:44  sondberg
## Added some POD (just a starting-point).
##
## Revision 1.1  2003/06/12 11:30:22  sondberg
## Moved around.
##
## Revision 1.5  2003/06/11 11:49:48  sondberg
## Multiple slashes are replaced by a single slash.
##
## Revision 1.4  2002/11/28 13:07:54  sondberg
## Global replacements of invalid entities.
##
## Revision 1.3  2002/11/28 13:00:15  sondberg
## Correct handling of entities.
##
## Revision 1.2  2002/11/27 15:00:17  sondberg
## Corrected a bug in find_portal_root.
##
## Revision 1.1  2002/11/27 14:52:31  sondberg
## Added this file to repository.
##
